import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Input,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import { MatPaginator } from '@angular/material/paginator';
import { MatTableDataSource } from '@angular/material/table';
import { fuseAnimations } from '@fuse/animations';
import { MemberBankDepositService } from 'app/modules/polyglot/member_bank_deposit/services/member_bank_deposit.service';
import { ListMemberBankDepositsRequest } from 'app/modules/proto/c2se/member_bank_deposit_pb';
import {
  MemberBankDepositModel,
  MemberBankDepositState,
} from 'app/modules/proto/models/member_bank_deposit_pb';
import { MemberModel } from 'app/modules/proto/models/member_pb';
import { Subject } from 'rxjs';

@Component({
  selector: 'deposit-history',
  templateUrl: './deposit-history.component.html',
  styles: [
    /* language=SCSS */
    `
      .deposit-history-grid {
        grid-template-columns: 60px auto 40px;

        @screen sm {
          grid-template-columns: 100px auto 60px 60px;
        }

        @screen md {
          grid-template-columns: 100px 100px auto 60px 60px;
        }

        @screen lg {
          grid-template-columns: 100px 100px 100px 60px 60px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class DepositHistoryComponent
  implements OnInit, AfterViewInit, OnDestroy
{
  @Input()
  member!: MemberModel | undefined;

  @ViewChild(MatPaginator) private _paginator!: MatPaginator;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  depositHistoryTableColumns: string[] = ['no', 'amount', 'createAt', 'state'];

  depositHistoryDataSource: MatTableDataSource<MemberBankDepositModel> =
    new MatTableDataSource();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _memberBankDepositService: MemberBankDepositService
  ) {
    /* this.depositHistoryDataSource.data = DEPOSIT_HISTORY_DATA; */
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    const req = new ListMemberBankDepositsRequest();
    const search = new ListMemberBankDepositsRequest.Search();

    search.setMemberId(this.member!.getId());
    req.setSearch(search);

    this._memberBankDepositService
      .listMemberBankDeposits(req)
      .then((result) => {
        const memberBankDepositList = result.getMemberBankDepositsList();
        this.depositHistoryDataSource.data = memberBankDepositList;

        this._changeDetectorRef.markForCheck();
      });
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {}

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  __getDepositState(deposit: MemberBankDepositModel): string | undefined {
    const state = deposit.getState();
    let result: string = '';

    switch (state) {
      case MemberBankDepositState.APPLICATION:
        result = '신청';
        break;
      case MemberBankDepositState.PENDING:
        result = '대기';
        break;
      case MemberBankDepositState.COMPLETE:
        result = '완료';
        break;
      default:
        break;
    }

    return result;
  }
}
